from typing import Callable, List
import threading
import logging

from . import fxcorepy


class ResponseListener(fxcorepy.AO2GResponseListener):
    """The class implements the AO2GResponseListener abstract class and calls the passed functions on request completions/failures and table updates."""
    def __init__(self,
                 session: fxcorepy.O2GSession,
                 on_request_completed_callback: Callable[[str, fxcorepy.O2GResponse], bool] = None,
                 on_request_failed_callback: Callable[[str, str], bool] = None,
                 on_tables_updates_callback: Callable[[fxcorepy.O2GResponse], None] = None) -> None:
        """ The constructor.
        
            Parameters
            ----------
            session : O2GSession
                An instance of O2GSession.
            on_request_completed_callback : typing.Callable[[str, O2GResponse], bool]
                The function that is called when a notification about a request completion is received.
            on_request_failed_callback : typing.Callable[[str, str], bool]
                The function that is called when a notification about a request failure is received.
            on_tables_updates_callback : typing.Callable[O2GResponse, None]
                The function that is called when a notification about a table update is received.
        
            Returns
            -------
            None
        
        """
        super(ResponseListener, self).__init__()
        self._on_request_completed_callback = on_request_completed_callback
        self._on_request_failed_callback = on_request_failed_callback
        self._on_tables_updates_callback = on_tables_updates_callback
        self._response = None
        self._error = None
        self._event = threading.Event()
        self._request_ids = None
        self._session = session

    @property
    def is_set(self) -> bool:
        """Reserved for future use."""
        return self._event.is_set()

    @property
    def response(self) -> fxcorepy.O2GResponse:
        """ Gets an instance of O2GResponse if it has been received.
        
            Returns
            -------
            O2GResponse
        
        """
        return self._response

    @property
    def error(self) -> str:
        """ Gets a string representation of an error that occurred when processing a request.
        
            Returns
            -------
            str
        
        """
        return self._error

    @property
    def session(self) -> fxcorepy.O2GSession:
        """ Gets a string representation of an error that occurred when processing a request.
        
            Returns
            -------
            str
        
        """
        return self._session

    def set_request_id(self, request_id: str) -> None:
        """Reserved for future use."""
        self.set_request_ids([request_id])

    def on_request_completed(self,
                             request: str,
                             response: fxcorepy.O2GResponse) -> None:  # native call
        """ Implements the method AO2GEachRowListener.on_request_completed and calls the function that processes notifications about the successful request completion. The function is passed in the constructor.
        
            Returns
            -------
            
        
        """
        logging.debug("Request completed %s", request)
        if request in self._request_ids:
            self._response = response
            result = None
            if self._on_request_completed_callback:
                result = self._on_request_completed_callback(request, response)
            self._request_ids.remove(request)
            if not self._request_ids and (result is None or (isinstance(result, bool) and result)):
                self.stop_waiting()

    def on_request_failed(self, request: str,
                          error: str) -> None:  # native call
        """Reserved for future use."""
        logging.error("Request failed %s: %s", request, error)
        if request in self._request_ids:
            self._error = error
            self._request_ids.remove(request)
            result = None
            if self._on_request_failed_callback:
                result = self._on_request_failed_callback(request, error)

            if not self._request_ids and (result is None or (isinstance(result, bool) and result)):
                self.stop_waiting()

    def on_tables_updates(self, response: fxcorepy.O2GResponse) -> None:  # native call
        """ Implements the method AO2GEachRowListener.on_tables_updates and calls the function that processes notifications about table updates. The function is passed in the constructor.
        
            Returns
            -------
            
        
        """
        if self._on_tables_updates_callback:
            self._on_tables_updates_callback(response)

    def set_request_ids(self, request_ids: List[str]) -> None:
        """Reserved for future use."""
        self._request_ids = request_ids
        self.reset()

    def wait_event(self) -> bool:
        """Reserved for future use."""
        return self._event.wait(30)

    def stop_waiting(self) -> None:
        """ Stops waiting for a response.
        
            Returns
            -------
            None
        
        """
        self._event.set()

    def reset(self) -> None:
        """ Resets the response listener after a response is received.
        
            Returns
            -------
            None
        
        """
        self._error = None
        self._response = None
        self._event.clear()


class ResponseListenerAsync(fxcorepy.AO2GResponseListener):
    """"""
    def __init__(self, fc) -> None:
        """Reserved for future use."""
        super(ResponseListenerAsync, self).__init__()
        self._another_listener = []
        self._fc = fc

    def add_response_listener(self, another_listener: ResponseListener) -> None:
        """ 
        
            Parameters
            ----------
            another_listener : ResponseListener
                
        
            Returns
            -------
            None
        
        """

        if another_listener not in self._another_listener:
            self._another_listener.append(another_listener)

    def remove_response_listener(self, another_listener: ResponseListener):
        """ 
        
            Parameters
            ----------
            another_listener : ResponseListener
                
        
            Returns
            -------
            None
        
        """
        if another_listener not in self._another_listener:
            return
        self._another_listener.remove(another_listener)

    def on_request_completed(self,
                             request: str,
                             response: fxcorepy.O2GResponse) -> None:  # native call
        """Reserved for future use."""
        if len(self._another_listener) == 0:
            return
        for listener in self._another_listener:
            listener.on_request_completed(request, response)
            if listener.is_set:
                self.remove_response_listener(listener)

    def on_request_failed(self, request: str,
                          error: str) -> None:  # native call
        """Reserved for future use."""
        if len(self._another_listener) == 0:
            return
        for listener in self._another_listener:
            listener.on_request_failed(request, error)
            if listener.is_set:
                self.remove_response_listener(listener)

    def on_tables_updates(self, response: fxcorepy.O2GResponse) -> None:  # native call
        """Reserved for future use."""
        if len(self._another_listener) == 0:
            return
        for listener in self._another_listener:
            listener.on_tables_updates(response)
            if listener.is_set:
                self.remove_response_listener(listener)




# import os
# import sys
# sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

# from JGTCore import env,jsonfile2prop,d2p
# # from JGTCDS createByRange
# import pandas as pd
# from functools import reduce

# import jgtetl

# ctx=d2p(env)

# trainsetfnjson=env['trainfnjson'] 
# trainsetfncsv=env['trainfncsv'] 

# dtlast = ""
# if hasattr(ctx, 'dtlast'):
#   dtlast=ctx.dtlast
#   print(dtlast)
# else:
#   dst=jsonfile2prop(trainsetfnjson)
#   l=0
#   for key, value in dst.items():
#     l=key
#   lastitem=d2p(dst[l])
#   # print(lastitem)
#   # print(lastitem.state)
#   # print(lastitem.plx)
#   # print(lastitem.bs)
#   dtlast= lastitem.dt
#   print('export lastdt="' + lastitem.dt+ '"')

# print("The variable, dtlast is of type:", type(dtlast))
# print("The variable, ctx.firstst is of type:", type(ctx.firstst))
# dtfirst=ctx.firstst
# dtfirst_with_offset=jgtetl.svc_offset_dt_by_tf(dtfirst,ctx.timeframe)

# # jgtetl.offsetdt
# #pto_nbhourbytfofoffset2212
# # ctx['lastdt']=lastitem.dt
# print("CReating CDS with Range: " + dtfirst_with_offset + " > " + dtlast)
# df=createByRange(ctx.instrument,ctx.timeframe,dtfirst_with_offset,dtlast)
# fnids=ctx.tsfnbase+".ids.csv"
# df.to_csv(fnids)

# #  SEE PART 02

# # fnbase=ctx.tsfnbase
# # # fn=fnbase + ".trainset.csv"
# # fn=trainsetfncsv

# # fnfdbb=fnbase + ".fdb."+ctx.bs+".csv"
# # fnplx=fnbase + ".plx.csv"
# # fnrrstotc=fnbase + ".rrstotc.csv"
# # #df=pd.DataFrame()
# # df=pd.read_csv(fn)
# # df.index.rename('no',inplace=True)
# # dfsel=pd.DataFrame()
# # fdbTag='FDBB'
# # if ctx.bs=='Sell':
# #   fdbTag='FDBS'
# # dfsel=df[df['sig']== fdbTag]
# # dfsel.to_csv(fnfdbb)
# # dfplx=dfsel[['bp','state','plx','plxc','tpl','tplc','to','tc','bs','dt','plxdt','i','tdt','oid','tid','r','ro','rs','rsx','sig','tf','tm']]
# # totplx=dfplx['plx'].sum()
# # print(totplx)
# # dfrrstotc=dfsel[['bp','state','plx','tpl','r','to','rs','tc','ro','plxc','tplc','bs','dt','plxdt','i','tdt','oid','tid','rsx','sig','tf','tm']]
# # #dfsel.index.rename('bp',inplace=True)
# # dfplx.to_csv(fnplx)
# # dfrrstotc.to_csv(fnrrstotc)



#!/usr/bin/env python
"""
jgtfxcon
"""

from setuptools import find_packages, setup

from jgtfxcon import version


INSTALL_REQUIRES = [
    "pandas>=0.25.1",
    "python-dotenv>=0.19.2",
    "jgtutils>=0.1.82",
    "jgtfx2console>=0.4.35",
    "tlid",
    "flask",
]

EXTRAS_DEV_LINT = [
    "flake8>=3.6.0,<3.7.0",
    "isort>=4.3.4,<4.4.0",
]

EXTRAS_DEV_TEST = [
    "coverage",
    "pytest>=3.10",
]

EXTRAS_DEV_DOCS = [
    "readme_renderer",
    "sphinx",
    "sphinx_rtd_theme>=0.4.0",
]

setup(
    name="jgtfxcon",
    version=version,
    description="JGTrading get data from fxconnect Dataframes",
    long_description=open("README.rst").read(),
    author="GUillaume Isabelle",
    author_email="jgi@jgwill.com",
    url="https://github.com/jgwill/jgtfxcon",
    packages=find_packages(
        include=[
            "jgtfxcon",
            "jgtfxcon.common_samples",
            "jgtfxcon.forexconnect",
            "jgtfxcon.forexconnect.lib",
            "jgtfxcon.forexconnect.lib.windows",
            "jgtfxcon.forexconnect.lib.linux",
            "jgtfxcon/**",
        ],
        exclude=["*test*"],
    ),
    include_package_data=True,
    install_requires=INSTALL_REQUIRES,
    # entry_points={
    #     "console_scripts": ["jgtfxcli=jgtfxcon.jgtfxcli:main"],
    # },
    extras_require={
        "dev": (EXTRAS_DEV_LINT + EXTRAS_DEV_TEST + EXTRAS_DEV_DOCS),
        "dev-lint": EXTRAS_DEV_LINT,
        "dev-test": EXTRAS_DEV_TEST,
        "dev-docs": EXTRAS_DEV_DOCS,
    },
    #license="MIT",
    keywords="data",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Financial and Insurance Industry",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3.7.16",
    ],
)
